@@ -70,7 +70,7 @@ def get_coalesce_output_type(args: list[Any]) -> CONCRETE_TYPES:
 
 
 def get_transform_output_type(args: list[Any]) -> CONCRETE_TYPES:
-    return arg_to_datatype(args[2])
+    return ArrayType(type=arg_to_datatype(args[2]))
 
 
 def get_index_output_type(
@@ -92,7 +92,7 @@ def get_attr_datatype(
     lookup = args[1]
     datatype = arg_to_datatype(arg)
     if isinstance(datatype, StructType):
-        return arg_to_datatype(datatype.fields_map[lookup])
+        return datatype.field_types[lookup]
     return datatype
 
 
@@ -178,6 +178,20 @@ def get_date_trunc_output(
         raise InvalidSyntaxException(f"Date truncation not supported for {target}")
 
 
+def get_map_key_type(arg):
+    arg_datatype = arg_to_datatype(arg)
+    if isinstance(arg_datatype, MapType):
+        return ArrayType(type=arg_datatype.key_data_type)
+    return ArrayType(type=DataType.STRING)
+
+
+def get_map_value_type(arg):
+    arg_datatype = arg_to_datatype(arg)
+    if isinstance(arg_datatype, MapType):
+        return ArrayType(type=arg_datatype.value_data_type)
+    return ArrayType(type=DataType.STRING)
+
+
 FUNCTION_REGISTRY: dict[FunctionType, FunctionConfig] = {
     FunctionType.ALIAS: FunctionConfig(
         arg_count=1,
@@ -254,6 +268,22 @@ FUNCTION_REGISTRY: dict[FunctionType, FunctionConfig] = {
         output_type_function=get_index_output_type,
         arg_count=2,
     ),
+    FunctionType.MAP_KEYS: FunctionConfig(
+        valid_inputs={
+            DataType.MAP,
+        },
+        output_purpose=Purpose.PROPERTY,
+        output_type_function=lambda args: get_map_key_type(args[0]),
+        arg_count=1,
+    ),
+    FunctionType.MAP_VALUES: FunctionConfig(
+        valid_inputs={
+            DataType.MAP,
+        },
+        output_purpose=Purpose.PROPERTY,
+        output_type_function=lambda args: get_map_value_type(args[0]),
+        arg_count=1,
+    ),
     FunctionType.ARRAY_DISTINCT: FunctionConfig(
         valid_inputs={
             DataType.ARRAY,
@@ -905,6 +935,7 @@ class FunctionFactory:
                 output_purpose = Purpose.METRIC
             else:
                 output_purpose = Purpose.PROPERTY
+
         return Function(
             operator=operator,
             arguments=full_args,  # type: ignore
