@@ -164,6 +164,26 @@ class ConceptRef(Addressable, Namespaced, DataTyped, Mergeable, BaseModel):
     def with_reference_replacement(self, source: str, target: Expr | ArgBinding):
         if self.address == source:
             return target
+
+        # a reference might be to an attribute of a struct that is bound late
+        # if the replacement is a parent in the access path; replace reference
+        # with an attribute access call
+        candidates = [f"local.{self.address}", self.address]
+        for candidate in candidates:
+            if not candidate.startswith(f"{source}."):
+                continue
+            attribute = self.address.rsplit(".", 1)[1]
+            dtype = arg_to_datatype(target)
+            if not isinstance(dtype, StructType):
+                continue
+            output_type = dtype.field_types.get(attribute, DataType.UNKNOWN)
+            return Function(
+                arguments=[target, self.address.rsplit(".", 1)[1]],
+                operator=FunctionType.ATTR_ACCESS,
+                arg_count=2,
+                output_datatype=output_type,
+                output_purpose=Purpose.PROPERTY,
+            )
         return self
 
 
@@ -440,6 +460,8 @@ class HavingClause(WhereClause):
 class Grain(Namespaced, BaseModel):
     components: set[str] = Field(default_factory=set)
     where_clause: Optional["WhereClause"] = None
+    _str: str | None = None
+    _abstract: bool = False
 
     def without_condition(self):
         return Grain(components=self.components)
@@ -464,12 +486,9 @@ class Grain(Namespaced, BaseModel):
         from trilogy.parsing.common import concepts_to_grain_concepts
 
         x = Grain.model_construct(
-            components={
-                c.address
-                for c in concepts_to_grain_concepts(
-                    concepts, environment=environment, local_concepts=local_concepts
-                )
-            },
+            components=concepts_to_grain_concepts(
+                concepts, environment=environment, local_concepts=local_concepts
+            ),
             where_clause=where_clause,
         )
 
@@ -530,17 +549,22 @@ class Grain(Namespaced, BaseModel):
             where_clause=self.where_clause,
         )
 
-    @property
-    def abstract(self):
+    def _gen_abstract(self) -> bool:
         return not self.components or all(
             [c.endswith(ALL_ROWS_CONCEPT) for c in self.components]
         )
 
+    @property
+    def abstract(self):
+        if not self._abstract:
+            self._abstract = self._gen_abstract()
+        return self._abstract
+
     def __eq__(self, other: object):
         if isinstance(other, list):
-            if not all([isinstance(c, Concept) for c in other]):
-                return False
-            return self.components == set([c.address for c in other])
+            if all([isinstance(c, Concept) for c in other]):
+                return self.components == set([c.address for c in other])
+            return False
         if not isinstance(other, Grain):
             return False
         if self.components == other.components:
@@ -561,15 +585,20 @@ class Grain(Namespaced, BaseModel):
         intersection = self.components.intersection(other.components)
         return Grain(components=intersection)
 
-    def __str__(self):
+    def _gen_str(self) -> str:
         if self.abstract:
             base = "Grain<Abstract>"
         else:
-            base = "Grain<" + ",".join([c for c in sorted(list(self.components))]) + ">"
+            base = "Grain<" + ",".join(sorted(self.components)) + ">"
         if self.where_clause:
             base += f"|{str(self.where_clause)}"
         return base
 
+    def __str__(self):
+        if not self._str:
+            self._str = self._gen_str()
+        return self._str
+
     def __radd__(self, other) -> "Grain":
         if other == 0:
             return self
@@ -637,33 +666,37 @@ class Comparison(ConceptArgs, Mergeable, DataTyped, Namespaced, BaseModel):
 
     @model_validator(mode="after")
     def validate_comparison(self):
+        left_type = arg_to_datatype(self.left)
+        right_type = arg_to_datatype(self.right)
+        left_name = (
+            left_type.name if isinstance(left_type, DataType) else str(left_type)
+        )
+        right_name = (
+            right_type.name if isinstance(right_type, DataType) else str(right_type)
+        )
         if self.operator in (ComparisonOperator.IS, ComparisonOperator.IS_NOT):
-            if self.right != MagicConstants.NULL and DataType.BOOL != arg_to_datatype(
-                self.right
-            ):
+            if self.right != MagicConstants.NULL and DataType.BOOL != right_type:
                 raise SyntaxError(
                     f"Cannot use {self.operator.value} with non-null or boolean value {self.right}"
                 )
         elif self.operator in (ComparisonOperator.IN, ComparisonOperator.NOT_IN):
-            right_type = arg_to_datatype(self.right)
+
             if isinstance(right_type, ArrayType) and not is_compatible_datatype(
-                arg_to_datatype(self.left), right_type.value_data_type
+                left_type, right_type.value_data_type
             ):
                 raise SyntaxError(
-                    f"Cannot compare {arg_to_datatype(self.left)} and {right_type} with operator {self.operator} in {str(self)}"
+                    f"Cannot compare {left_type} and {right_type} with operator {self.operator} in {str(self)}"
                 )
             elif isinstance(self.right, Concept) and not is_compatible_datatype(
-                arg_to_datatype(self.left), arg_to_datatype(self.right)
+                left_type, right_type
             ):
                 raise SyntaxError(
-                    f"Cannot compare {arg_to_datatype(self.left)} and {arg_to_datatype(self.right)} with operator {self.operator} in {str(self)}"
+                    f"Cannot compare {left_name} and {right_name} with operator {self.operator} in {str(self)}"
                 )
         else:
-            if not is_compatible_datatype(
-                arg_to_datatype(self.left), arg_to_datatype(self.right)
-            ):
+            if not is_compatible_datatype(left_type, right_type):
                 raise SyntaxError(
-                    f"Cannot compare {arg_to_datatype(self.left)} and {arg_to_datatype(self.right)} of different types with operator {self.operator} in {str(self)}"
+                    f"Cannot compare {left_name} ({self.left}) and {right_name} ({self.right}) of different types with operator {self.operator.value} in {str(self)}"
                 )
 
         return self
@@ -869,7 +902,11 @@ class Concept(Addressable, DataTyped, ConceptArgs, Mergeable, Namespaced, BaseMo
 
     @property
     def is_aggregate(self):
-        return self.calculate_is_aggregate(self.lineage)
+        base = getattr(self, "_is_aggregate", None)
+        if base:
+            return base
+        setattr(self, "_is_aggregate", self.calculate_is_aggregate(self.lineage))
+        return self._is_aggregate
 
     def with_merge(self, source: Self, target: Self, modifiers: List[Modifier]) -> Self:
         if self.address == source.address:
@@ -1036,18 +1073,25 @@ class Concept(Addressable, DataTyped, ConceptArgs, Mergeable, Namespaced, BaseMo
         final_grain = grain if not self.grain.components else self.grain
         keys = self.keys
 
-        if self.is_aggregate and isinstance(new_lineage, Function) and grain.components:
+        if self.is_aggregate and grain.components and isinstance(new_lineage, Function):
             grain_components: list[ConceptRef | Concept] = [
                 environment.concepts[c].reference for c in grain.components
             ]
-            new_lineage = AggregateWrapper(function=new_lineage, by=grain_components)
+            new_lineage = AggregateWrapper.model_construct(
+                function=new_lineage, by=grain_components
+            )
             final_grain = grain
             keys = set(grain.components)
-        elif isinstance(new_lineage, AggregateWrapper) and not new_lineage.by and grain:
+        elif (
+            grain
+            and new_lineage
+            and isinstance(new_lineage, AggregateWrapper)
+            and not new_lineage.by
+        ):
             grain_components = [
                 environment.concepts[c].reference for c in grain.components
             ]
-            new_lineage = AggregateWrapper(
+            new_lineage = AggregateWrapper.model_construct(
                 function=new_lineage.function, by=grain_components
             )
             final_grain = grain
@@ -1100,7 +1144,7 @@ class Concept(Addressable, DataTyped, ConceptArgs, Mergeable, Namespaced, BaseMo
             granularity=self.granularity,
             derivation=self.derivation,
             lineage=self.lineage,
-            grain=grain if grain else Grain(components=set()),
+            grain=grain if grain else Grain.model_construct(components=set()),
             namespace=self.namespace,
             keys=self.keys,
             modifiers=self.modifiers,
@@ -1624,6 +1668,9 @@ class Function(DataTyped, ConceptArgs, Mergeable, Namespaced, BaseModel):
     ] = None
     arguments: Sequence[FuncArgs]
 
+    class Config:
+        frozen = True
+
     def __repr__(self):
         return f'{self.operator.value}({",".join([str(a) for a in self.arguments])})'
 
@@ -1732,8 +1779,16 @@ class Function(DataTyped, ConceptArgs, Mergeable, Namespaced, BaseModel):
         ]
         if self.output_datatype == DataType.UNKNOWN:
             new_output = merge_datatypes([arg_to_datatype(x) for x in nargs])
+
+            if self.operator == FunctionType.ATTR_ACCESS:
+                if isinstance(new_output, StructType):
+                    new_output = new_output.field_types[str(nargs[1])]
         else:
             new_output = self.output_datatype
+        # this is not ideal - see hacky logic for datatypes above
+        # we need to figure out how to patch properly
+        # should use function factory, but does not have environment access
+        # probably move all datatype resolution to build?
         return Function.model_construct(
             operator=self.operator,
             arguments=nargs,
@@ -1792,17 +1847,6 @@ class Function(DataTyped, ConceptArgs, Mergeable, Namespaced, BaseModel):
             base += get_concept_arguments(arg)
         return base
 
-    @property
-    def output_grain(self):
-        # aggregates have an abstract grain
-        base_grain = Grain(components=[])
-        if self.operator in FunctionClass.AGGREGATE_FUNCTIONS.value:
-            return base_grain
-        # scalars have implicit grain of all arguments
-        for input in self.concept_arguments:
-            base_grain += input.grain
-        return base_grain
-
 
 class FunctionCallWrapper(
     DataTyped,
@@ -2290,11 +2334,16 @@ class AlignItem(Namespaced, BaseModel):
 
 class CustomFunctionFactory:
     def __init__(
-        self, function: Expr, namespace: str, function_arguments: list[ArgBinding]
+        self,
+        function: Expr,
+        namespace: str,
+        function_arguments: list[ArgBinding],
+        name: str,
     ):
         self.namespace = namespace
         self.function = function
         self.function_arguments = function_arguments
+        self.name = name
 
     def with_namespace(self, namespace: str):
         self.namespace = namespace
@@ -2319,7 +2368,27 @@ class CustomFunctionFactory:
             for binding in self.function_arguments[len(creation_arg_list) :]:
                 if binding.default is None:
                     raise ValueError(f"Missing argument {binding.name}")
+
                 creation_arg_list.append(binding.default)
+        for arg_idx, arg in enumerate(self.function_arguments):
+            if not arg.datatype or arg.datatype == DataType.UNKNOWN:
+                continue
+            if arg_idx > len(creation_arg_list):
+                continue
+            comparison = arg_to_datatype(creation_arg_list[arg_idx])
+            if comparison != arg.datatype:
+                raise TypeError(
+                    f"Invalid type passed into custom function @{self.name} in position {arg_idx+1} for argument {arg.name}, expected {arg.datatype}, got {comparison}"
+                )
+            if isinstance(arg.datatype, TraitDataType):
+                if not (
+                    isinstance(comparison, TraitDataType)
+                    and all(x in comparison.traits for x in arg.datatype.traits)
+                ):
+                    raise TypeError(
+                        f"Invalid argument type passed into custom function @{self.name} in position {arg_idx+1} for argument {arg.name}, expected traits {arg.datatype.traits}, got {comparison}"
+                    )
+
         if isinstance(nout, Mergeable):
             for idx, x in enumerate(creation_arg_list):
                 if self.namespace == DEFAULT_NAMESPACE:
@@ -2431,6 +2500,7 @@ FuncArgs = (
     | CaseElse
     | WindowItem
     | FilterItem
+    | bool
     | int
     | float
     | DatePart
