@@ -27,7 +27,7 @@ from pydantic_core import core_schema
 from trilogy.constants import (
     MagicConstants,
 )
-from trilogy.core.enums import Ordering
+from trilogy.core.enums import Modifier, Ordering
 
 
 class DataTyped(ABC):
@@ -211,9 +211,31 @@ class MapType(BaseModel):
         return self.key_type
 
 
+class StructComponent(BaseModel):
+    name: str
+    type: TYPEDEF_TYPES
+    modifiers: list[Modifier] = []
+
+    @field_validator("type", mode="plain")
+    def validate_Type(cls, v):
+        return v
+
+
 class StructType(BaseModel):
-    fields: Sequence[TYPEDEF_TYPES]
-    fields_map: Dict[str, DataTyped | int | float | str]
+    fields: Sequence[StructComponent | TYPEDEF_TYPES]
+    fields_map: Dict[str, DataTyped | int | float | str | StructComponent]
+
+    def __repr__(self):
+        return "struct<{}>".format(
+            ", ".join(
+                f"{field.name}:{field.type.name}"
+                for field in self.fields
+                if isinstance(field, StructComponent)
+            )
+        )
+
+    def __str__(self) -> str:
+        return self.__repr__()
 
     @field_validator("fields", mode="plain")
     def validate_type(cls, v):
@@ -234,13 +256,30 @@ class StructType(BaseModel):
     def value(self):
         return self.data_type.value
 
+    @property
+    def field_types(self) -> Dict[str, CONCRETE_TYPES]:
+        out: Dict[str, CONCRETE_TYPES] = {}
+        keys = list(self.fields_map.keys())
+        for idx, field in enumerate(self.fields):
+            if isinstance(field, StructComponent):
+                out[field.name] = arg_to_datatype(field.type)
+            elif isinstance(field, DataTyped):
+                out[keys[idx]] = field.output_datatype
+            else:
+                out[keys[idx]] = field
+        return out
+
+    def __hash__(self):
+        return hash(str(self))
+
 
 class ListWrapper(Generic[VT], UserList):
     """Used to distinguish parsed list objects from other lists"""
 
-    def __init__(self, *args, type: DataType, **kwargs):
+    def __init__(self, *args, type: DataType, nullable: bool = False, **kwargs):
         super().__init__(*args, **kwargs)
         self.type = type
+        self.nullable = nullable
 
     @classmethod
     def __get_pydantic_core_schema__(
@@ -289,10 +328,11 @@ class MapWrapper(Generic[KT, VT], UserDict):
 class TupleWrapper(Generic[VT], tuple):
     """Used to distinguish parsed tuple objects from other tuples"""
 
-    def __init__(self, val, type: DataType, **kwargs):
+    def __init__(self, val, type: DataType, nullable: bool = False, **kwargs):
         super().__init__()
         self.type = type
         self.val = val
+        self.nullable = nullable
 
     def __getnewargs__(self):
         return (self.val, self.type)
@@ -318,15 +358,19 @@ class TupleWrapper(Generic[VT], tuple):
 
 
 def list_to_wrapper(args):
-    types = [arg_to_datatype(arg) for arg in args]
-    assert len(set(types)) == 1
-    return ListWrapper(args, type=types[0])
+    rtypes = [arg_to_datatype(arg) for arg in args]
+    types = [arg for arg in rtypes if arg != DataType.NULL]
+    if not len(set(types)) == 1:
+        raise SyntaxError(f"Cannot create a list with this set of types: {set(types)}")
+    return ListWrapper(args, type=types[0], nullable=DataType.NULL in rtypes)
 
 
 def tuple_to_wrapper(args):
-    types = [arg_to_datatype(arg) for arg in args]
-    assert len(set(types)) == 1
-    return TupleWrapper(args, type=types[0])
+    rtypes = [arg_to_datatype(arg) for arg in args]
+    types = [arg for arg in rtypes if arg != DataType.NULL]
+    if not len(set(types)) == 1:
+        raise SyntaxError(f"Cannot create a tuple with this set of types: {set(types)}")
+    return TupleWrapper(args, type=types[0], nullable=DataType.NULL in rtypes)
 
 
 def dict_to_map_wrapper(arg):
@@ -401,6 +445,8 @@ def arg_to_datatype(arg) -> CONCRETE_TYPES:
         return DataType.STRING
     elif isinstance(arg, float):
         return DataType.FLOAT
+    elif isinstance(arg, DataType):
+        return arg
     elif isinstance(arg, NumericType):
         return arg
     elif isinstance(arg, TraitDataType):
@@ -420,6 +466,8 @@ def arg_to_datatype(arg) -> CONCRETE_TYPES:
         return DataType.DATETIME
     elif isinstance(arg, date):
         return DataType.DATE
+    elif isinstance(arg, StructComponent):
+        return arg_to_datatype(arg.type)
     else:
         raise ValueError(
             f"Cannot parse arg datatype for arg of raw type {type(arg)} value {arg}"
