@@ -859,7 +859,7 @@ class RecursiveCTE(CTE):
                     ),
                     BuildCaseElse(expr=False),
                 ],
-                output_datatype=DataType.BOOL,
+                output_data_type=DataType.BOOL,
                 output_purpose=Purpose.KEY,
             ),
         )
@@ -884,7 +884,7 @@ class RecursiveCTE(CTE):
                     ),
                     BuildCaseElse(expr=False),
                 ],
-                output_datatype=DataType.BOOL,
+                output_data_type=DataType.BOOL,
                 output_purpose=Purpose.KEY,
             ),
         )
@@ -909,7 +909,7 @@ class RecursiveCTE(CTE):
                     ),
                     BuildCaseElse(expr=right_recurse_concept),
                 ],
-                output_datatype=recursive_derived.datatype,
+                output_data_type=recursive_derived.datatype,
                 output_purpose=recursive_derived.purpose,
             ),
         )
@@ -1076,6 +1076,11 @@ class UnionCTE(BaseModel):
     def group_to_grain(self) -> bool:
         return False
 
+    @property
+    def group_concepts(self) -> List[BuildConcept]:
+        # unions should always be on unique sets
+        return []
+
     def __add__(self, other):
         if not isinstance(other, UnionCTE) or not other.name == self.name:
             raise SyntaxError("Cannot merge union CTEs")
