@@ -89,6 +89,7 @@ from trilogy.core.models.core import (
     MapType,
     MapWrapper,
     NumericType,
+    StructComponent,
     StructType,
     TraitDataType,
     TupleWrapper,
@@ -160,6 +161,11 @@ class WholeGrainWrapper:
     where: WhereClause
 
 
+@dataclass
+class FunctionBindingType:
+    type: DataType | TraitDataType | None = None
+
+
 with open(join(dirname(__file__), "trilogy.lark"), "r") as f:
     PARSER = Lark(
         f.read(),
@@ -238,7 +244,7 @@ def unwrap_transformation(
     elif isinstance(input, Parenthetical):
         return unwrap_transformation(input.content, environment)
     else:
-        return Function(
+        return Function.model_construct(
             operator=FunctionType.CONSTANT,
             output_datatype=arg_to_datatype(input),
             output_purpose=Purpose.CONSTANT,
@@ -443,20 +449,39 @@ class ParseToObjects(Transformer):
     def MINUS(self, args) -> str:
         return "-"
 
+    @v_args(meta=True)
+    def struct_component(self, meta: Meta, args) -> StructComponent:
+        modifiers = []
+        for arg in args:
+            if isinstance(arg, Modifier):
+                modifiers.append(arg)
+        return StructComponent(name=args[0], type=args[1], modifiers=modifiers)
+
     @v_args(meta=True)
     def struct_type(self, meta: Meta, args) -> StructType:
         final: list[
-            DataType | MapType | ArrayType | NumericType | StructType | Concept
+            DataType
+            | MapType
+            | ArrayType
+            | NumericType
+            | StructType
+            | StructComponent
+            | Concept
         ] = []
         for arg in args:
-            new = self.environment.concepts.__getitem__(  # type: ignore
-                key=arg, line_no=meta.line
-            )
-            final.append(new)
+            if isinstance(arg, StructComponent):
+                final.append(arg)
+            else:
+                new = self.environment.concepts.__getitem__(  # type: ignore
+                    key=arg, line_no=meta.line
+                )
+                final.append(new)
 
         return StructType(
             fields=final,
-            fields_map={x.name: x for x in final if isinstance(x, Concept)},
+            fields_map={
+                x.name: x for x in final if isinstance(x, (Concept, StructComponent))
+            },
         )
 
     def list_type(self, args) -> ArrayType:
@@ -681,9 +706,12 @@ class ParseToObjects(Transformer):
                 environment=self.environment,
                 metadata=metadata,
             )
+
             # let constant purposes exist to support round-tripping
             # as a build concept may end up with a constant based on constant inlining happening recursively
-            if (
+            if purpose == Purpose.KEY and concept.purpose != Purpose.KEY:
+                concept.purpose = Purpose.KEY
+            elif (
                 purpose
                 and purpose != Purpose.AUTO
                 and concept.purpose != purpose
@@ -900,7 +928,6 @@ class ParseToObjects(Transformer):
             name=output,
             metadata=metadata,
         )
-
         return ConceptTransform(function=transformation, output=concept)
 
     @v_args(meta=True)
@@ -1351,11 +1378,24 @@ class ParseToObjects(Transformer):
     def function_binding_list(self, meta: Meta, args) -> list[ArgBinding]:
         return args
 
+    @v_args(meta=True)
+    def function_binding_type(self, meta: Meta, args) -> FunctionBindingType:
+        return FunctionBindingType(type=args[0])
+
+    @v_args(meta=True)
+    def function_binding_default(self, meta: Meta, args):
+        return args[1]
+
     @v_args(meta=True)
     def function_binding_item(self, meta: Meta, args) -> ArgBinding:
-        if len(args) == 2:
-            return ArgBinding.model_construct(name=args[0], default=args[1])
-        return ArgBinding.model_construct(name=args[0], default=None)
+        default = None
+        type = None
+        for arg in args[1:]:
+            if isinstance(arg, FunctionBindingType):
+                type = arg.type
+            else:
+                default = arg
+        return ArgBinding.model_construct(name=args[0], datatype=type, default=default)
 
     @v_args(meta=True)
     def raw_function(self, meta: Meta, args) -> FunctionDeclaration:
@@ -1367,6 +1407,7 @@ class ParseToObjects(Transformer):
             function=output,
             namespace=self.environment.namespace,
             function_arguments=function_arguments,
+            name=identity,
         )
         return FunctionDeclaration(name=identity, args=function_arguments, expr=output)
 
@@ -2018,12 +2059,31 @@ class ParseToObjects(Transformer):
 
     @v_args(meta=True)
     def fnot(self, meta, args):
+        if arg_to_datatype(args[0]) == DataType.BOOL:
+            return Comparison(
+                left=self.function_factory.create_function(
+                    [args[0], False], FunctionType.COALESCE, meta
+                ),
+                operator=ComparisonOperator.EQ,
+                right=False,
+                meta=meta,
+            )
         return self.function_factory.create_function(args, FunctionType.IS_NULL, meta)
 
     @v_args(meta=True)
     def fbool(self, meta, args):
         return self.function_factory.create_function(args, FunctionType.BOOL, meta)
 
+    @v_args(meta=True)
+    def fmap_keys(self, meta, args):
+        return self.function_factory.create_function(args, FunctionType.MAP_KEYS, meta)
+
+    @v_args(meta=True)
+    def fmap_values(self, meta, args):
+        return self.function_factory.create_function(
+            args, FunctionType.MAP_VALUES, meta
+        )
+
     @v_args(meta=True)
     def farray_sum(self, meta, args):
         return self.function_factory.create_function(args, FunctionType.ARRAY_SUM, meta)
